
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.0.3'
version = '0.0.3'
full_version = '0.0.3'
git_revision = '1a8de7b3dfa7476abecb972122ff83abc4ba9974'
release = True

if not release:
    version = full_version
